#
######################################################
#
#  SaVi by Robert Thurman (thurman@geom.umn.edu) and
#          Patrick Worfolk (worfolk@alum.mit.edu).
#
#  Copyright (c) 1997 by The Geometry Center.
#  This file is part of SaVi.  SaVi is free software;
#  you can redistribute it and/or modify it only under
#  the terms given in the file COPYRIGHT which you should
#  have received along with this file.  SaVi may be
#  obtained by emailing savi@geom.umn.edu.
#
######################################################
#
# coverage.tcl
#
# $Id: coverage.tcl,v 1.1.1.1 2001/11/11 19:59:57 lloydwood Exp $

#
# define the size of the image
#

#
# Note: IM_W and IM_H are linked to C variables for the image size
#

set im_x_offset 2
set im_y_offset 2

proc coverage(build) {} {
    global coverage_flag coverage_rect_flag coverage_angle_flag \
	coverage_angle IM_W IM_H im_x_offset im_y_offset \
	tracks_flag project_tracks_flag FONT COLOR


    if [winfo exists .coverage] {
	wm deiconify .coverage
	raise .coverage
	return
    }

    set name [build_Toplevel coverage]

    build_Title $name "View of global map and ground coverage"

    set cmd [build_CmdFrame $name cmd]

    set c "$cmd.c"
    canvas $c -bg black -width $IM_W -height $IM_H
    pack $c
    set im [image create photo im_coverage -width $IM_W -height $IM_H]
    $c create image [expr $IM_W/2+$im_x_offset] [expr $IM_H/2+$im_y_offset] \
	-image $im

    # catch button click in map to get fisheye Lat/Long
    bind $c <Button-1> {coverage(lat_long) %x %y}

    build_IPopupMenu $cmd p0 "Map projection used is" \
        coverage_rect_flag coverage(map) {"sinusoidal" "cylindrical"}

    set cmda [build_StdFrame $cmd a]
    build_Optionslist $cmda olist \
	{"Show coverage" coverage_flag} \
	{"Add interval decay" no_access_flag} \
	{"Show ground tracks" tracks_flag}
# this will give a numerical idea of coverage by latitude
#       {"Print access data" print_access_flag} 
# this is hacked into utils.tcl instead. 
#	{"Project ground tracks for the next..." project_tracks_flag}
    pack $cmda.olist -side left

    build_StdFrame $cmda ce
    # the background colors should match the colors in coverage_vis.c
    build_StdFrame $cmda.ce cf
    label $cmda.ce.cf.0 -text "diversity of coverage now:" \
	    -font $FONT(label) -bg $COLOR(bg)
    label $cmda.ce.cf.1 -text "1" -font $FONT(fixed) -bg #ffff00
    label $cmda.ce.cf.2 -text "2" -font $FONT(fixed) -bg #ff7f00
    label $cmda.ce.cf.3 -text "3" -font $FONT(fixed) -bg #ff2f00
    label $cmda.ce.cf.4 -text "4+" -font $FONT(fixed) -bg #ff0000
    label $cmda.ce.cf.5 -text "satellites" -font $FONT(label) -bg $COLOR(bg)
    pack $cmda.ce.cf.0 $cmda.ce.cf.1 $cmda.ce.cf.2 $cmda.ce.cf.3 $cmda.ce.cf.4 \
         $cmda.ce.cf.5 -expand 1 -fill x -side left -ipadx 2m -ipady 2m \
	 -padx 1m -pady 1m
    pack $cmda.ce.cf -side top -fill x

    build_StdFrame $cmda.ce cg
    label $cmda.ce.cg.0 -text "age of decayed coverage:" -font $FONT(label) -bg $COLOR(bg)
    label $cmda.ce.cg.1 -text "1" -font $FONT(fixed) -bg #0000ff
    label $cmda.ce.cg.2 -text "2" -font $FONT(fixed) -bg #0000d9
    label $cmda.ce.cg.3 -text "3" -font $FONT(fixed) -bg #0000b4
    label $cmda.ce.cg.4 -text "4+" -font $FONT(fixed) -bg #00008f
    label $cmda.ce.cg.5 -text "intervals" -font $FONT(label) -bg $COLOR(bg)
    pack $cmda.ce.cg.0 $cmda.ce.cg.1 $cmda.ce.cg.2 $cmda.ce.cg.3 $cmda.ce.cg.4 \
         $cmda.ce.cg.5 -expand 1 -fill x -side left -ipadx 2m -ipady 2m \
	 -padx 1m -pady 1m
    pack $cmda.ce.cg -fill y

    pack $cmda.ce -side top

    pack $cmda -fill x -expand 1

    build_LabelEntryColumns $cmd le2 \
	{lcheckbox "" project_tracks_flag} \
	{ientry "" {tracks_interval}} \
	{text "" {hours}}

    build_IPopupMenu $cmd p1 "Coverage angle describes:" \
	coverage_angle_flag coverage(foot) {"mask elevation" "satellite cone"}

    build_LabelEntryColumns $cmd le1 \
	{text "" "angle:"} \
	{dentry "" {coverage_angle}} \
	{label "" {coverage}} \
	{label "" {mean_coverage}}


    build_LabelEntryColumns $cmd le0 \
	{label "" {coverage_init_time}} \
	{label "" {coverage_time}} \
	{text "" "Time:"} \
	{label "" {time}}

    pack $cmd -fill x

    # an explicit Dismiss Window button halts computation.
    build_Buttonbar $name dbbar \
	{"Refresh" coverage(foot)} \
	{"Dismiss window" coverage(dismiss)}

    coverage(update)
    update
}

proc coverage(dismiss) {} {
    global coverage_flag tracks_flag no_access_flag

    # turn off flags to avoid computing drawing when window not shown
    set coverage_flag 0
    set no_access_flag 0
    set tracks_flag 0
    destroy .coverage
}

proc coverage(update) {} {
    global coverage_flag

    if {![winfo exists .coverage]} return
    set flag $coverage_flag
    set coverage_flag 1
    if {$flag == 0} {
	set coverage_flag 0
    }
}

proc coverage(lat_long) {x y} {
    global Fisheye coverage_rect_flag IM_W IM_H im_x_offset im_y_offset

    set x [expr $x-$im_x_offset]
    set y [expr $y-$im_y_offset]

    set HALF_X [expr $IM_W/2]
    set HALF_Y [expr $IM_H/2]

    if {$coverage_rect_flag == 0} {
	set lat [expr ($HALF_Y-$y)/double($HALF_Y)*90.0]
	if {abs($lat) == 90} {
	    set long 0
	} else {
	    set long [expr \
		    ($x-$HALF_X)/double($HALF_X)/cos($lat/180.0*3.14159)*180.0-90.0]
	}
	if {$long > 90} {
	    set long 90
	} elseif {$long < -270} {
	    set long -270
	}
    } else {
	set long [expr ($x-$HALF_X)/double($IM_W)*360.0-90.0]
	set y [expr 1-$y/double($IM_H)*2]
	if {$y < -1} {
	    set y -1
	}
	set lat [expr asin($y)/3.1416*180.0]
    }

    if {$long <= -180} {
	set long [expr $long+360.0]
    }

    set Fisheye(lat_degrees) [expr {$lat<0 ? -int(-$lat) : int($lat)} ]
    set Fisheye(lat_minutes) [expr abs($lat-$Fisheye(lat_degrees))*60.0]
    set Fisheye(long_degrees) [expr {$long<0 ? -int(-$long) : int($long)} ]
    set Fisheye(long_minutes) [expr abs($long-$Fisheye(long_degrees))*60.0]
    fisheye(apply)

}

proc coverage(foot) {} {
    coverage(update)
}

proc coverage(map) {} {
    coverage(update)
}
