#
######################################################
#
#  SaVi by Robert Thurman (thurman@geom.umn.edu) and
#          Patrick Worfolk (worfolk@alum.mit.edu).
#
#  Copyright (c) 1997 by The Geometry Center.
#  This file is part of SaVi.  SaVi is free software;
#  you can redistribute it and/or modify it only under
#  the terms given in the file COPYRIGHT which you should
#  have received along with this file.  SaVi may be
#  obtained by emailing savi@geom.umn.edu.
#
######################################################
#
# params.tcl
#
# $Id: params.tcl,v 1.1.1.1 2001/11/11 19:59:57 lloydwood Exp $


proc params(build) {} {

    if [winfo exists .params] {
	wm deinconfiy .params
	raise .params
	return
    }

    set name [build_Toplevel params]
    build_Title $name "Simulation constants"

    set cmd [build_CmdFrame $name cmd]

    build_LabelEntryColumns $cmd le0 \
	    {text " " {"Radius (km)" "Rotation rate (rad/s)" } } \
	    {dentry "Central body" {tparams(Radius) tparams(Omega) } }

    build_LabelEntryColumns $cmd le1 \
	{text " " {"Mu (km^3/s^2)" "Oblateness (J2)"} } \
	{dentry "Gravitational model" {tparams(Mu) tparams(J2) } }

    pack $cmd -fill both -expand 1

    build_DismissButtonbar $name dbbar "destroy .params" \
	{"Apply" params(apply)} \
	{"Cancel" params(cancel)} \
	{"Reset to default" params(reset)}

    update
    params(cancel)
}

proc params(apply) {} {
    global params tparams

    set inds [array names tparams]
    foreach ind $inds {
	set params($ind) $tparams($ind)
    }

    # now update other panels if necessary
    edit(update)
}

proc params(cancel) {} {
    global params tparams

    set inds [array names tparams]
    foreach ind $inds {
	set tparams($ind) $params($ind)
    }
}

proc params(reset) {} {
    satellites RESET_PARAMS
    params(cancel)
}




